use super::*;
use std::{fs, io, os::unix::fs::DirBuilderExt, path::PathBuf};

pub trait MultiDomain: MailStore {
    type Error;
    fn domains(&self) -> Result<Vec<String>, Self::Error>;
    fn add_domain(&mut self, domain: &str) -> Result<(), Self::Error>;
    fn remove_domain(&mut self, domain: &str) -> Result<(), Self::Error>;
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
pub struct Filesystem {
    pub path: PathBuf,
}

impl MailStore for Filesystem {
    fn users(&self) -> Vec<Mailuser> {
        todo!()
    }

    fn serves_domain(&self, domain: &str) -> bool {
        let mut path = self.path.clone();
        path.push(domain);
        path.exists()
    }

    fn has_mailuser(&self, mailuser: &str) -> bool {
        if let Some((user, host)) = mailuser.rsplit_once('@') {
            let mut path = self.path.clone();
            if host.contains('.') && !host.contains(|c: char| c.is_whitespace()) {
                path.push(host);
                path.push(user);
                return path.exists();
            }
        }
        false
    }

    fn get_user_messages(&self, user: &str) -> Option<BTreeMap<String, Message>> {
        todo!()
    }

    fn get_message(&self, user: &str, title: &str) -> Option<Message> {
        todo!()
    }

    fn move_message(&mut self, user: &str, title: &str, folder: &str) {
        todo!()
    }

    fn add_user(&mut self, user: &str) -> Result<(), ParseMailboxError> {
        todo!()
    }

    fn remove_user(&mut self, user: &str) -> bool {
        todo!()
    }
}

impl MultiDomain for Filesystem {
    type Error = io::Error;

    fn domains(&self) -> Result<Vec<String>, Self::Error> {
        Ok(self
            .path
            .read_dir()?
            .filter(|x| {
                if let Ok(x) = x {
                    if let Ok(t) = x.file_type() {
                        return t.is_dir();
                    }
                }
                false
            })
            .map(|x| x.unwrap().file_name().to_string_lossy().to_string())
            .collect::<Vec<String>>())
    }

    fn add_domain(&mut self, domain: &str) -> Result<(), Self::Error> {
        let mut path = self.path.clone();
        path.push(domain);
        if !path.exists() {
            fs::DirBuilder::new()
                .recursive(true)
                .mode(700)
                .create(path)?;
        }
        Ok(())
    }

    fn remove_domain(&mut self, domain: &str) -> Result<(), Self::Error> {
        let mut path = self.path.clone();
        path.push(domain);
        if path.exists() {
            fs::remove_dir_all(path)?;
        }
        Ok(())
    }
}
